@@ -25,12 +25,11 @@
 use std::cmp::max;
 use std::{fmt, mem};
 
-use crate::ansi::graphics::MAX_GRAPHIC_DIMENSIONS;
 use crate::config::colors::ColorRgb;
-use sugarloaf::{ColorType, GraphicData, GraphicId};
+use sugarloaf::{ColorType, GraphicData, GraphicId, MAX_GRAPHIC_DIMENSIONS};
 
 use copa::Params;
-use log::trace;
+use tracing::trace;
 
 /// Type for color registers.
 #[derive(Copy, Clone, Default, Debug, PartialEq, Eq)]
@@ -48,27 +47,31 @@ const MAX_COMMAND_PARAMS: usize = 5;
 #[derive(Debug)]
 pub enum Error {
     /// Image dimensions are too big.
-    TooBigImage { width: usize, height: usize },
+    TooBigImage {
+        width: usize,
+        height: usize,
+    },
 
     /// A component in a color introducer is not valid.
-    InvalidColorComponent { register: u16, component_value: u16 },
+    InvalidColorComponent {
+        register: u16,
+        component_value: u16,
+    },
 
     /// The coordinate system to define the color register is not valid.
     InvalidColorCoordinateSystem {
         register: u16,
         coordinate_system: u16,
     },
+
+    NonExistentParser,
 }
 
 impl fmt::Display for Error {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             Error::TooBigImage { width, height } => {
-                write!(
-                    fmt,
-                    "The image dimensions are too big ({}, {})",
-                    width, height
-                )
+                write!(fmt, "The image dimensions are too big ({width}, {height})")
             }
 
             Error::InvalidColorComponent {
@@ -77,8 +80,7 @@ impl fmt::Display for Error {
             } => {
                 write!(
                     fmt,
-                    "Invalid color component {} for register {}",
-                    component_value, register
+                    "Invalid color component {component_value} for register {register}"
                 )
             }
 
@@ -88,10 +90,13 @@ impl fmt::Display for Error {
             } => {
                 write!(
                     fmt,
-                    "Invalid color coordinate system {} for register {}",
-                    coordinate_system, register
+                    "Invalid color coordinate system {coordinate_system} for register {register}"
                 )
             }
+
+            Error::NonExistentParser => {
+                write!(fmt, "Parser does not exist",)
+            }
         }
     }
 }
@@ -543,6 +548,7 @@ impl Parser {
             color_type: ColorType::Rgba,
             pixels: rgba_pixels,
             is_opaque,
+            resize: None,
         };
 
         Ok((data, self.color_registers))
@@ -707,11 +713,7 @@ mod tests {
 
         // Reimplement abs_diff to be compatible with rustc before 1.60.
         fn abs_diff(x: u8, y: u8) -> u8 {
-            if x > y {
-                x - y
-            } else {
-                y - x
-            }
+            x.abs_diff(y)
         }
 
         macro_rules! assert_color {
