@@ -113,7 +113,7 @@ pub enum Event<T: 'static> {
     },
 
     OpenConfig,
-    HookEvent(Hook),
+    HookEvent(KeyEvent, Modifiers),
 }
 
 impl<T> Event<T> {
@@ -132,19 +132,11 @@ impl<T> Event<T> {
             MemoryWarning => Ok(MemoryWarning),
             Opened { urls } => Ok(Opened { urls }),
             OpenConfig => Ok(OpenConfig),
-            HookEvent(hook) => Ok(HookEvent(hook)),
+            HookEvent(hook, modifiers) => Ok(HookEvent(hook, modifiers)),
         }
     }
 }
 
-#[derive(Debug, Clone, PartialEq)]
-pub enum Hook {
-    CreateTab,
-    CloseTab,
-    Copy,
-    Paste,
-}
-
 /// Describes the reason the event loop is resuming.
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum StartCause {
@@ -588,11 +580,11 @@ pub struct KeyEvent {
     /// ## Caveats
     ///
     /// - Certain niche hardware will shuffle around physical key positions, e.g. a keyboard that
-    ///     implements DVORAK in hardware (or firmware)
+    ///   implements DVORAK in hardware (or firmware)
     /// - Your application will likely have to handle keyboards which are missing keys that your
-    ///     own keyboard has.
+    ///   own keyboard has.
     /// - Certain `KeyCode`s will move between a couple of different positions depending on what
-    ///     layout the keyboard was manufactured to support.
+    ///   layout the keyboard was manufactured to support.
     ///
     ///  **Because of these caveats, it is important that you provide users with a way to configure
     ///  most (if not all) keybinds in your application.**
@@ -615,7 +607,7 @@ pub struct KeyEvent {
     /// This has two use cases:
     /// - Allows querying whether the current input is a Dead key.
     /// - Allows handling key-bindings on platforms which don't
-    ///     support [`key_without_modifiers`].
+    ///   support [`key_without_modifiers`].
     ///
     /// If you use this field (or [`key_without_modifiers`] for that matter) for keyboard
     /// shortcuts, **it is important that you provide users with a way to configure your
@@ -624,7 +616,7 @@ pub struct KeyEvent {
     ///
     /// ## Platform-specific
     /// - **Web:** Dead keys might be reported as the real key instead
-    ///     of `Dead` depending on the browser/OS.
+    ///   of `Dead` depending on the browser/OS.
     ///
     /// [`key_without_modifiers`]: crate::platform::modifier_supplement::KeyEventExtModifierSupplement::key_without_modifiers
     pub logical_key: keyboard::Key,
@@ -1239,7 +1231,7 @@ mod tests {
     #[test]
     fn ensure_attrs_do_not_panic() {
         foreach_event!(|event: event::Event<()>| {
-            let _ = format!("{:?}", event);
+            let _ = format!("{event:?}");
         });
         let _ = event::StartCause::Init.clone();
 
