@@ -1,14 +1,18 @@
 use super::cache::ImageCache;
 use super::{AddImage, ImageData, ImageId, ImageLocation};
 use crate::font::FontLibrary;
-use crate::font_introspector::scale::{
-    image::{Content, Image as GlyphImage},
-    *,
-};
 use crate::font_introspector::zeno::Format;
+use crate::font_introspector::{
+    scale::{
+        image::{Content, Image as GlyphImage},
+        *,
+    },
+    FontRef,
+};
 use core::borrow::Borrow;
 use core::hash::{Hash, Hasher};
 use rustc_hash::FxHashMap;
+use tracing::debug;
 use zeno::{Angle, Transform};
 
 // const IS_MACOS: bool = cfg!(target_os = "macos");
@@ -61,11 +65,6 @@ impl GlyphCache {
         }
     }
 
-    #[inline]
-    pub fn set_max_height(&mut self, max_height: u16) {
-        self.max_height = max_height;
-    }
-
     // pub fn prune(&mut self, images: &mut ImageCache) {
     //     self.fonts.retain(|_, entry| {
     //         for glyph in &entry.glyphs {
@@ -107,7 +106,7 @@ pub struct GlyphCacheSession<'a> {
     max_height: &'a u16,
 }
 
-impl<'a> GlyphCacheSession<'a> {
+impl GlyphCacheSession<'_> {
     pub fn get_image(&mut self, image: ImageId) -> Option<ImageLocation> {
         self.images.get(&image)
     }
@@ -124,17 +123,30 @@ impl<'a> GlyphCacheSession<'a> {
             }
         }
 
+        // Log cache miss for debugging
+        debug!(
+            "GlyphCache miss for glyph_id={} size={} font={}",
+            id, self.quant_size, self.font
+        );
+
         self.scaled_image.data.clear();
-        let mut font_library_data = self.font_library.inner.lock();
+        let font_library_data = self.font_library.inner.read();
         let enable_hint = font_library_data.hinting;
         let font_data = font_library_data.get(&self.font);
         let should_embolden = font_data.should_embolden;
         let should_italicize = font_data.should_italicize;
 
-        if let Some(data) = font_library_data.get_data(&self.font) {
+        if let Some((shared_data, offset, cache_key)) =
+            font_library_data.get_data(&self.font)
+        {
+            let font_ref = FontRef {
+                data: shared_data.as_ref(),
+                offset,
+                key: cache_key,
+            };
             let mut scaler = self
                 .scale_context
-                .builder(data)
+                .builder(font_ref)
                 // With the advent of high-DPI displays (displays with >300 pixels per inch),
                 // font hinting has become less relevant, as aliasing effects become
                 // un-noticeable to the human eye.
@@ -148,8 +160,7 @@ impl<'a> GlyphCacheSession<'a> {
 
             // let embolden = if IS_MACOS { 0.25 } else { 0. };
             if Render::new(SOURCES)
-                .format(Format::CustomSubpixel([0.3, 0., -0.3]))
-                // .format(Format::Alpha)
+                .format(Format::Alpha)
                 // .offset(Vector::new(subpx[0].to_f32(), subpx[1].to_f32()))
                 .embolden(if should_embolden { 0.5 } else { 0.0 })
                 .transform(if should_italicize {
@@ -165,11 +176,52 @@ impl<'a> GlyphCacheSession<'a> {
                 let p = self.scaled_image.placement;
                 let w = p.width as u16;
                 let h = p.height as u16;
+
+                // Handle zero-sized glyphs (spaces, zero-width characters) efficiently
+                if w == 0 || h == 0 {
+                    let entry = GlyphEntry {
+                        left: p.left,
+                        top: p.top,
+                        width: w,
+                        height: h,
+                        image: ImageId::empty(), // Use a special empty image ID
+                        is_bitmap: false,
+                    };
+                    self.entry.glyphs.insert(key, entry);
+                    return Some(entry);
+                }
+
+                // Use the appropriate content type and data format
+                let (image_data, content_type) = match self.scaled_image.content {
+                    Content::Mask => {
+                        // Alpha format: use data directly for R8 texture
+                        (
+                            ImageData::Borrowed(&self.scaled_image.data),
+                            super::ContentType::Mask,
+                        )
+                    }
+                    Content::Color => {
+                        // Already RGBA format
+                        (
+                            ImageData::Borrowed(&self.scaled_image.data),
+                            super::ContentType::Color,
+                        )
+                    }
+                    Content::SubpixelMask => {
+                        // Subpixel format (should not happen with Format::Alpha)
+                        (
+                            ImageData::Borrowed(&self.scaled_image.data),
+                            super::ContentType::Color,
+                        )
+                    }
+                };
+
                 let req = AddImage {
                     width: w,
                     height: h,
                     has_alpha: true,
-                    data: ImageData::Borrowed(&self.scaled_image.data),
+                    data: image_data,
+                    content_type,
                 };
                 let image = self.images.allocate(req)?;
 
@@ -237,7 +289,7 @@ enum Coords<'a> {
     Ref(&'a [i16]),
 }
 
-impl<'a> Coords<'a> {
+impl Coords<'_> {
     fn new(coords: &[i16]) -> Self {
         let len = coords.len();
         if len == 0 {
@@ -252,7 +304,7 @@ impl<'a> Coords<'a> {
     }
 }
 
-impl<'a> Coords<'a> {
+impl Coords<'_> {
     fn as_ref(&self) -> &[i16] {
         match self {
             Self::None => &[],
