@@ -1,7 +1,6 @@
 #[cfg(target_os = "macos")]
 pub fn external_fallbacks() -> Vec<String> {
     vec![
-        // String::from(".SF NS"),
         String::from("Menlo"),
         String::from("Geneva"),
         String::from("Arial Unicode MS"),
@@ -25,8 +24,18 @@ pub fn external_fallbacks() -> Vec<String> {
 #[cfg(not(any(target_os = "macos", windows)))]
 pub fn external_fallbacks() -> Vec<String> {
     vec![
+        /* Sans-serif fallbacks */
         String::from("Noto Sans"),
+        /* More sans-serif fallbacks */
+        String::from("DejaVu Sans"),
         String::from("FreeSans"),
+        /* Mono fallbacks */
+        String::from("Noto Sans Mono"),
+        String::from("DejaVu Sans Mono"),
+        String::from("FreeMono"),
+        /* Symbols fallbacks */
+        String::from("Noto Sans Symbols"),
+        String::from("Noto Sans Symbols2"),
         // String::from("Noto Color Emoji"),
     ]
 }
